/* $Id: messagebox.cc,v 1.3 2002/05/26 14:41:00 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001-2002 Richard Atterer
  | \/|  <richard@atterer.net>
   '` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Display an error box with a message and an OK button.

*/

#include <config.h>
#include <debug.hh>
#include <gtk-gui.hh>
#include <gtk-support.hh>
#include <messagebox.hh>
//______________________________________________________________________

GUI::Errorbox GUI::errorbox;

namespace {
  struct ErrorBox {
    GtkWidget* errorbox;
  };
}

void on_errorbox_okButton_clicked(GtkButton*, gpointer p) {
  ErrorBox* e = static_cast<ErrorBox*>(p);
  //cerr << "errorBox " << e << ": OK" << endl;
  gtk_widget_destroy(e->errorbox);
  delete e;
}

void on_errorbox_cancelButton_clicked(GtkButton*, gpointer p) {
  ErrorBox* e = static_cast<ErrorBox*>(p);
  //cerr << "errorBox " << e << ": Cancel" << endl;
  gtk_widget_destroy(e->errorbox);
  delete e;
}

void on_errorbox_abortButton_clicked(GtkButton*, gpointer p) {
  ErrorBox* e = static_cast<ErrorBox*>(p);
  //cerr << "errorBox " << e << ": Abort" << endl;
  gtk_widget_destroy(e->errorbox);
  delete e;
}

gboolean on_errorbox_delete_event(GtkWidget*, GdkEvent*, gpointer p) {
  ErrorBox* e = static_cast<ErrorBox*>(p);
  //cerr << "errorBox " << e << ": Closed" << endl;
  delete e;
  return FALSE; // destroy error box
}
//______________________________________________________________________

namespace {

inline void addButtons(GtkWidget* window, ErrorBox* e, GtkWidget* hButtonBox,
                       unsigned buttons) {
  using namespace MessageBox;
  bool first = true;
  //GtkAccelGroup* accelGroup = gtk_accel_group_new();

  for (unsigned mask = 1; mask <= MAX_BUTTON; mask = mask << 1) {
    GtkWidget* pixmap;
    void (*func)(GtkButton*, gpointer);
    const char* text;
    switch (buttons & mask) {
    case 0:
      continue;
    case OK:
      pixmap = create_pixmap(window, "button-play.xpm");
      func = &on_errorbox_okButton_clicked;
      text = _("OK");
      break;
    case CANCEL:
      pixmap = create_pixmap(window, "button-cross.xpm");
      func = &on_errorbox_cancelButton_clicked;
      text = _("Cancel");
      break;
    case ABORT:
      pixmap = create_pixmap(window, "button-cross.xpm");
      func = &on_errorbox_abortButton_clicked;
      text = _("Abort");
      break;
    default:
      Paranoid(false);
      continue;
    }


    GtkWidget* button = gtk_button_new();
    gtk_widget_ref(button);
    gtk_container_add(GTK_CONTAINER(hButtonBox), button);
    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

    GtkWidget* hBox = gtk_hbox_new(FALSE, 4);
    gtk_widget_ref(hBox);
    gtk_widget_show(hBox);
    gtk_container_add(GTK_CONTAINER(button), hBox);
    gtk_container_set_border_width(GTK_CONTAINER(hBox), 2);

    gtk_widget_ref(pixmap);
    gtk_widget_show(pixmap);
    gtk_box_pack_start(GTK_BOX(hBox), pixmap, FALSE, FALSE, 0);

    GtkWidget* label = gtk_label_new(text);
    //GtkWidget* label = gtk_label_new("");
    //guint labelKey = gtk_label_parse_uline(GTK_LABEL(label), text);
    gtk_widget_ref(label);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hBox), label, FALSE, FALSE, 0);
    //gtk_widget_add_accelerator(button, "clicked", accelGroup,
    //                           labelKey, GDK_MOD1_MASK, (GtkAccelFlags)0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(func),
                       static_cast<gpointer>(e));

    if (first) {
      gtk_widget_grab_default(button);
      first = false;
    }
    gtk_widget_show(button);
  }
  //gtk_window_add_accel_group(GTK_WINDOW(window), accelGroup);
}

} // namespace
//______________________________________________________________________

void MessageBox::error(const char* message, unsigned buttons) {
  ErrorBox* e = new ErrorBox();
  Paranoid(GUI::errorbox.errorbox == 0);
  GUI::errorbox.create();
  e->errorbox = GUI::errorbox.errorbox;
  addButtons(e->errorbox, e, GUI::errorbox.hButtonBox, buttons);
  gtk_label_set_text(GTK_LABEL(GUI::errorbox.message), message);
  gtk_widget_show(GUI::errorbox.errorbox);
  gtk_signal_connect(GTK_OBJECT(e->errorbox), "delete_event",
                     GTK_SIGNAL_FUNC(on_errorbox_delete_event),
                     static_cast<gpointer>(e));
  GUI::errorbox.errorbox = 0;
}
